@@ -1,6 +1,6 @@
 package Module::Build::Base;
 
-# $Id: Base.pm,v 1.28 2002/07/11 13:56:30 ken Exp $
+# $Id: Base.pm,v 1.32 2002/08/23 08:45:11 ken Exp $
 
 use strict;
 use Config;
@@ -59,10 +59,19 @@ sub new {
 				  },
 		    new_cleanup => {},
 		   }, $package;
+  my $p = $self->{properties};
 
   # Synonyms
-  for ($self->{properties}) {
-    $_->{requires} = delete $_->{prereq} if exists $_->{prereq};
+  $p->{requires} = delete $p->{prereq} if exists $p->{prereq};
+
+  # Shortcuts
+  if (exists $p->{module_name}) {
+    ($p->{dist_name} = $p->{module_name}) =~ s/::/-/g
+      unless exists $p->{dist_name};
+    $p->{dist_version_from} = join( '/', 'lib', split '::', $p->{module_name} ) . '.pm'
+      unless exists $p->{dist_version_from} or exists $p->{dist_version};
+    
+    delete $p->{module_name};
   }
 
   $self->check_manifest;
@@ -91,8 +100,9 @@ sub resume {
   my %valid_properties = map {$_ => 1}
     qw(
        module_name
-       module_version
-       module_version_from
+       dist_name
+       dist_version
+       dist_version_from
        requires
        recommends
        PL_files
@@ -149,21 +159,17 @@ sub find_version {
   my ($self) = @_;
   my $p = $self->{properties};
 
-  return if exists $p->{module_version};
+  return if exists $p->{dist_version};
   
   my $version_from;
-  if (exists $p->{module_version_from}) {
-    # module_version_from is always a Unix-style path
-    $version_from = File::Spec->catfile( split '/', delete $p->{module_version_from} );
-  } elsif (exists $p->{module_name}) {
-    my $search_path = File::Spec->catdir($p->{base_dir}, 'lib');
-    $version_from = $self->find_module_by_name($p->{module_name}, [$search_path]);
-    die "Can't find '$p->{module_name}' in $search_path for version check" unless defined $version_from;
+  if (exists $p->{dist_version_from}) {
+    # dist_version_from is always a Unix-style path
+    $version_from = File::Spec->catfile( split '/', delete $p->{dist_version_from} );
   } else {
-    die "Must supply either 'module_version', 'module_version_from', or 'module_name' parameter";
+    die "Must supply either 'dist_version', 'dist_version_from', or 'module_name' parameter";
   }
 
-  $p->{module_version} = $self->version_from_file($version_from);
+  $p->{dist_version} = $self->version_from_file($version_from);
 }
 
 sub find_module_by_name {
@@ -250,32 +256,62 @@ sub write_config {
   File::Path::mkpath($self->{properties}{config_dir});
   -d $self->{properties}{config_dir} or die "Can't mkdir $self->{properties}{config_dir}: $!";
   
-  my $file = $self->config_file('build_params');
-  open my $fh, ">$file" or die "Can't create '$file': $!";
   local $Data::Dumper::Terse = 1;
+
+  my $file = $self->config_file('build_params');
+  open my $fh, "> $file" or die "Can't create '$file': $!";
   print $fh Data::Dumper::Dumper([$self->{args}, $self->{config}, $self->{properties}]);
   close $fh;
+
+  $file = $self->config_file('prereqs');
+  open $fh, "> $file" or die "Can't create '$file': $!";
+  my @items = qw(requires build_requires conflicts recommends);
+  print $fh Data::Dumper::Dumper( { map {$_,$self->{properties}{$_}} @items } );
+  close $fh;
+}
+
+sub prereq_failures {
+  my $self = shift;
+
+  my @types = qw(requires recommends build_requires conflicts);
+  my $out;
+
+  foreach my $type (@types) {
+    while (my ($modname, $spec) = each %{$self->{properties}{$type}}) {
+      my $status = $self->check_installed_status($modname, $spec);
+      
+      if ($type eq 'conflicts') {
+	next if !$status->{ok};
+	$status->{conflicts} = delete $status->{need};
+	$status->{message} = "Installed version '$status->{have}' of $modname conflicts with this distribution";
+      } else {
+	next if $status->{ok};
+      }
+
+      $out->{$type}{$modname} = $status;
+    }
+  }
+
+  return $out;
 }
 
 sub check_prereq {
   my $self = shift;
 
-  my $pass = 1;
-  while (my ($modname, $spec) = each %{$self->{properties}{requires}}) {
-    my $thispass = $self->check_installed_version($modname, $spec);
-    warn "WARNING: $@\n" unless $thispass;
-    $pass &&= $thispass;
+  my $failures = $self->prereq_failures;
+  return 1 unless $failures;
+  
+  foreach my $type (qw(requires build_requires conflicts recommends)) {
+    next unless $failures->{$type};
+    my $prefix = $type eq 'recommends' ? 'WARNING' : 'ERROR';
+    while (my ($module, $status) = each %{$failures->{$type}}) {
+      warn "$prefix: $module: $status->{message}\n";
+    }
   }
-
-  while (my ($modname, $spec) = each %{$self->{properties}{recommends}}) {
-    warn "NOTE: $@\n" unless $self->check_installed_version($modname, $spec);
-  }
-
-  if (!$pass) {
-    warn "ERRORS FOUND IN PREREQUISITES.  You may wish to install the versions ".
-         "of the modules indicated above before proceeding with this installation.\n";
-  }
-  return $pass;
+  
+  warn "ERRORS/WARNINGS FOUND IN PREREQUISITES.  You may wish to install the versions ".
+       "of the modules indicated above before proceeding with this installation.\n";
+  return 0;
 }
 
 sub perl_version_to_float {
@@ -285,28 +321,27 @@ sub perl_version_to_float {
   return $version;
 }
 
-# I wish I could set $! to a string, but I can't, so I use $@
-sub check_installed_version {
+sub check_installed_status {
   my ($self, $modname, $spec) = @_;
-
-  my $installed_version;
+  my %status = (need => $spec);
+  
   if ($modname eq 'perl') {
     # Check the current perl interpreter
     # It's much more convenient to use $] here than $^V, but 'man
     # perlvar' says I'm not supposed to.  Bloody tyrant.
-    $installed_version = $^V ? $self->perl_version_to_float(sprintf "%vd", $^V) : $];
+    $status{have} = $^V ? $self->perl_version_to_float(sprintf "%vd", $^V) : $];
     
   } else {
     my $file = $self->find_module_by_name($modname, \@INC);
     unless ($file) {
-      $@ = "Prerequisite $modname isn't installed";
-      return 0;
+      @status{ qw(have message) } = ('<none>', "Prerequisite $modname isn't installed");
+      return \%status;
     }
-
-    $installed_version = $self->version_from_file($file);
-    if ($spec and !$installed_version) {
-      $@ = "Couldn't find a \$VERSION in prerequisite '$file'";
-      return 0;
+    
+    $status{have} = $self->version_from_file($file);
+    if ($spec and !$status{have}) {
+      @status{ qw(have message) } = (undef, "Couldn't find a \$VERSION in prerequisite '$file'");
+      return \%status;
     }
   }
   
@@ -319,20 +354,35 @@ sub check_installed_version {
   
   foreach (@conditions) {
     unless ( /^\s*  (<=?|>=?|==|!=)  \s*  ([\w.]+)  \s*$/x ) {
-      $@ = "Invalid prerequisite condition for $modname: $_";
-      return 0;
+      die "Invalid prerequisite condition '$_' for $modname";
     }
     if ($modname eq 'perl') {
       my ($op, $version) = ($1, $2);
       $_ = "$op " . $self->perl_version_to_float($version);
     }
-    unless (eval "\$installed_version $_") {
-      $@ = "$modname version $installed_version is installed, but we need version $_";
-      return 0;
+    unless (eval "\$status{have} $_") {
+      $status{message} = "Version $status{have} is installed, but we need version $_";
+      return \%status;
     }
   }
+  
+  $status{ok} = 1;
+  return \%status;
+}
 
-  return $installed_version ? $installed_version : '0 but true';
+# I wish I could set $! to a string, but I can't, so I use $@
+sub check_installed_version {
+  my ($self, $modname, $spec) = @_;
+  
+  my $status = $self->check_installed_status($modname, $spec);
+  
+  if ($status->{ok}) {
+    return $status->{have} if $status->{have} and $status->{have} ne '<none>';
+    return '0 but true';
+  }
+  
+  $@ = $status->{message};
+  return 0;
 }
 
 sub rm_previous_build_script {
@@ -393,7 +443,7 @@ sub create_build_script {
   $self->rm_previous_build_script;
 
   print("Creating new '$self->{properties}{build_script}' script for ",
-	"'$self->{properties}{module_name}' version '$self->{properties}{module_version}'\n");
+	"'$self->{properties}{dist_name}' version '$self->{properties}{dist_version}'\n");
   open my $fh, ">$self->{properties}{build_script}" or die "Can't create '$self->{properties}{build_script}': $!";
   $self->print_build_script($fh);
   close $fh;
@@ -692,9 +742,7 @@ sub ACTION_manifest {
 
 sub dist_dir {
   my ($self) = @_;
-
-  (my $dist_dir = $self->{properties}{module_name}) =~ s/::/-/;
-  return "$dist_dir-$self->{properties}{module_version}";
+  return "$self->{properties}{dist_name}-$self->{properties}{dist_version}";
 }
 
 sub write_metadata {
@@ -709,8 +757,8 @@ sub write_metadata {
   my %metadata = (
 		  distribution_type => 'module',
 		  dynamic_config => 0,
-		  name => $p->{module_name},
-		  version => $p->{module_version},
+		  name => $p->{dist_name},
+		  version => $p->{dist_version},
 		  license => $p->{license},
 		  generated_by => (ref($self) || $self) . " version " . $self->VERSION,
 		 );
